@@ -29,13 +29,14 @@ public class DateCommands {
 				end_dateedit.Time = query.Range.End;
 			}
 
-			this.Dialog.DefaultResponse = ResponseType.Ok;
+			Dialog.TransientFor = parent_window;
+			Dialog.DefaultResponse = ResponseType.Ok;
 			ResponseType response = (ResponseType) this.Dialog.Run ();
 
 			bool success = false;
 
 			if (response == ResponseType.Ok) {
-				query.Range = new PhotoStore.DateRange (start_dateedit.Time, end_dateedit.Time);
+				query.Range = new PhotoStore.DateRange (start_dateedit.Time, end_dateedit.Time.Add(new System.TimeSpan(23,59,59)));
 				success = true;
 			}
 			
