@@ -3,6 +3,8 @@ using Gtk;
 using Gnome;
 using System.Collections;
 using System;
+using FSpot;
+using FSpot.Xmp;
 using System.IO;
 
 public class ImportException : System.Exception {
@@ -13,40 +15,63 @@ public class ImportException : System.Exception {
 
 public class FileImportBackend : ImportBackend {
 	PhotoStore store;
+	TagStore tag_store = FSpot.Core.Database.Tags;
 	bool recurse;
 	bool copy;
 	string [] base_paths;
 	Tag [] tags;
 
 	int count;
+	XmpTagsImporter xmptags;
 
-	ArrayList file_paths;
-	ArrayList imported_photos;
+	ArrayList import_info;
 	Stack directories;
+
+	private class ImportInfo {
+		string original_path;
+		public string destination_path;
+		public Photo Photo;
+	       
+		public string OriginalPath {
+			get { return original_path; }
+		}
+		
+		public string DestinationPath {
+			get { return destination_path; }
+			set { destination_path = value; }
+		}
+		
+		public ImportInfo (string original)
+		{
+			original_path = original;
+		        destination_path= null;
+			Photo = null;
+		}
+	}
 	
 	private void AddPath (string path)
 	{
 		if (FSpot.ImageFile.HasLoader (path))
-			file_paths.Add (path);
+			import_info.Add (new ImportInfo (path));
 	}
 
 	private void GetListing (System.IO.DirectoryInfo info)
 	{
 		try {
 			GetListing (info, info.GetFiles (), recurse);
-		} catch (System.UnauthorizedAccessException e) {
+		} catch (System.UnauthorizedAccessException) {
 			System.Console.WriteLine ("Unable to access directory {0}", info.FullName);
 		} catch (System.Exception e) {
 			System.Console.WriteLine ("{0}", e.ToString ());
 		}
 	}
 
-	private void GetListing (System.IO.DirectoryInfo info, System.IO.FileInfo [] files, bool recurse)
+	private void GetListing (System.IO.DirectoryInfo dirinfo, System.IO.FileInfo [] files, bool recurse)
 	{
-		System.Console.WriteLine ("Scanning {0}", info.FullName);
+		System.Console.WriteLine ("Scanning {0}", dirinfo.FullName);
 		Hashtable exiting_entries = new Hashtable ();
 
-		foreach (Photo p in store.Query (info)) {
+		foreach (Photo p in store.Query (dirinfo)) {
 			foreach (uint id in p.VersionIds) {
 				string name;
 				if (id == Photo.OriginalVersionId)
@@ -65,7 +90,7 @@ public class FileImportBackend : ImportBackend {
 		}
 
 		if (recurse) {
-			foreach (System.IO.DirectoryInfo d in info.GetDirectories ()){
+			foreach (System.IO.DirectoryInfo d in dirinfo.GetDirectories ()){
 				if (!d.Name.StartsWith ("."))
 					GetListing (d);
 			}
@@ -74,10 +99,10 @@ public class FileImportBackend : ImportBackend {
 
 	public override int Prepare ()
 	{
-		if (file_paths != null)
+		if (import_info != null)
 			throw new ImportException ("Busy");
 
-		file_paths = new ArrayList ();
+		import_info = new ArrayList ();
 
 		foreach (string path in base_paths) {
 			try {	
@@ -91,10 +116,10 @@ public class FileImportBackend : ImportBackend {
 			}
 		}	
 
-		imported_photos = new ArrayList ();
 		directories = new Stack ();
+		xmptags = new XmpTagsImporter (store, tag_store);
 
-		return file_paths.Count;
+		return import_info.Count;
 	}
 
 	public static string UniqueName (string path, string filename)
@@ -125,7 +150,7 @@ public class FileImportBackend : ImportBackend {
 		FSpot.ImageFile img = FSpot.ImageFile.Create (path);
 		DateTime time = img.Date;
 		
-		string dest_dir = String.Format ("{0}{1}{2}{1}{3}{1}{4}",
+		string dest_dir = String.Format ("{0}{1}{2}{1}{3:D2}{1}{4:D2}",
 						 FSpot.Global.PhotoDirectory,
 						 System.IO.Path.DirectorySeparatorChar,
 						 time.Year,
@@ -167,50 +192,57 @@ public class FileImportBackend : ImportBackend {
 
 	public override bool Step (out Photo photo, out Pixbuf thumbnail, out int count)
 	{
-		if (file_paths == null)
+		if (import_info == null)
 			throw new ImportException ("Prepare() was not called");
 
-		if (this.count == file_paths.Count)
+		if (this.count == import_info.Count)
 			throw new ImportException ("Already finished");
 
 		// FIXME Need to get the EXIF info etc.
-		string path = (string) file_paths [this.count];
-		
+		ImportInfo info = (ImportInfo)import_info [this.count];
+		bool needs_commit = false;
+
 		try {
-			string dest = path;
+			string destination = info.OriginalPath;
 			if (copy)
-				dest = ChooseLocation (path, directories);
+				destination = ChooseLocation (info.OriginalPath, directories);
 			
-			if (path == dest) {
-				photo = store.Create (path, out thumbnail);
+			// Don't copy if we are already home
+			if (info.OriginalPath == destination) {
+				info.DestinationPath = destination;
+				photo = store.Create (info.DestinationPath, out thumbnail);
 			} else {
-				System.IO.File.Copy (path, dest);
-				photo = store.Create (dest, path, out thumbnail);
-				
+				System.IO.File.Copy (info.OriginalPath, destination);
+				info.DestinationPath = destination;
+
+				photo = store.Create (info.DestinationPath, info.OriginalPath, out thumbnail);
+
 				try {
-					File.SetAttributes (dest, File.GetAttributes (dest) & ~FileAttributes.ReadOnly);
-					DateTime create = File.GetCreationTime (path);
-					File.SetCreationTime (dest, create);
-					DateTime mod = File.GetLastWriteTime (path);
-					File.SetLastWriteTime (dest, mod);
-				} catch (System.Exception e) {
+					File.SetAttributes (destination, File.GetAttributes (info.DestinationPath) & ~FileAttributes.ReadOnly);
+					DateTime create = File.GetCreationTime (info.OriginalPath);
+					File.SetCreationTime (info.DestinationPath, create);
+					DateTime mod = File.GetLastWriteTime (info.OriginalPath);
+					File.SetLastWriteTime (info.DestinationPath, mod);
+				} catch (IOException) {
 					// we don't want an exception here to be fatal.
 				}
-
-				path = dest;
-
 			} 
 
 			if (tags != null) {
 				foreach (Tag t in tags) {
 					photo.AddTag (t);
 				}
-				store.Commit(photo);
+				needs_commit = true;
 			}
-			imported_photos.Add (photo);
+
+			needs_commit |= xmptags.Import (photo, info.DestinationPath, info.OriginalPath);
+
+			if (needs_commit)
+				store.Commit(photo);
 			
+			info.Photo = photo;
 		} catch (System.Exception e) {
-			System.Console.WriteLine ("Error importing {0}\n{1}", path, e.ToString ());
+			System.Console.WriteLine ("Error importing {0}\n{1}", info.OriginalPath, e.ToString ());
 			thumbnail = null;
 			photo = null;
 		}
@@ -218,24 +250,28 @@ public class FileImportBackend : ImportBackend {
 		this.count ++;
 		count = this.count;
 
-		return count != file_paths.Count;
+		return count != import_info.Count;
 	}
 
 	public override void Cancel ()
 	{
-		if (imported_photos == null)
+		if (import_info == null)
 			throw new ImportException ("Not doing anything");
 
-		foreach (Photo p in imported_photos) {
-			if (copy) {
+		foreach (ImportInfo info in import_info) {
+			
+			if (info.OriginalPath != info.DestinationPath) {
 				try {
-					System.IO.File.Delete (p.DefaultVersionPath);
+					System.IO.File.Delete (info.DestinationPath);
+				} catch (System.ArgumentNullException) {
+					// Do nothing, since if DestinationPath == null, we do not have to remove it
 				} catch (System.Exception e) {
 					System.Console.WriteLine (e);
 				}
 			}
 			
-			store.Remove (p);
+			if (info.Photo != null)
+				store.Remove (info.Photo);
 		}
 		
 		// clean up all the directories we created.
@@ -250,20 +286,22 @@ public class FileImportBackend : ImportBackend {
 					info.Delete ();
 			}
 		}
+		// Clean up just created tags
+		xmptags.Cancel();
 	}
 
 	public override void Finish ()
 	{
-		if (file_paths == null)
+		if (import_info == null)
 			throw new ImportException ("Not doing anything");
 
-		file_paths = null;
-		
-		foreach (Photo p in imported_photos) {
-			FSpot.ThumbnailGenerator.Default.Request (p.DefaultVersionPath, 0, 256, 256);
+		foreach (ImportInfo info in import_info) {
+			if (info.Photo != null) 
+				FSpot.ThumbnailGenerator.Default.Request (info.Photo.DefaultVersionUri, 0, 256, 256);
 		}
 
-		imported_photos = null;
+		import_info = null;
+		xmptags.Finish();
 		count = 0;
 	}
 
