@@ -378,7 +378,7 @@ namespace FSpot {
 					md.Run ();
 					md.Destroy ();
 					return;
-				} catch (GalleryRemote.GalleryCommandException e) {
+				} catch (GalleryRemote.GalleryException e) {
 					HigMessageDialog md = 
 						new HigMessageDialog (Dialog, 
 								      Gtk.DialogFlags.Modal |
@@ -386,6 +386,10 @@ namespace FSpot {
 								      Gtk.MessageType.Error, Gtk.ButtonsType.Ok,
 								      Catalog.GetString ("Error while connecting to Gallery"),
 								      String.Format (Catalog.GetString ("The following error was encountered while attempting to log in: {0}"), e.Message));
+					if (e.ResponseText != null) {
+						System.Console.WriteLine (e.Message);
+						System.Console.WriteLine (e.ResponseText);
+					}
 					md.Run ();
 					md.Destroy ();
 					return;
@@ -677,7 +681,7 @@ namespace FSpot {
 				
 				FilterSet filters = new FilterSet ();
 				if (scale)
-					filters.Add (new ResizeFilter ((uint)size));
+					filters.Add (new ResizeFilter ((uint) size));
 				else if (rotate)
 					filters.Add (new OrientationFilter ());
 				
@@ -694,23 +698,28 @@ namespace FSpot {
 					progress_dialog.ProgressText = System.String.Format (Catalog.GetString ("{0} of {1}"), photo_index, items.Length);
 					
 					
-					string orig = item.DefaultVersionUri.LocalPath;
+					Filters.FilterRequest req = new Filters.FilterRequest (item.DefaultVersionUri);
 
-					// FIXME the filters need to be able to handle entension
-					// changes directly themselves.
-					string final = orig;
-					if (scale)
-						final = ImageFile.TempPath (orig, "jpg");
-					else if (rotate)
-						final = ImageFile.TempPath (orig);
-					
-					if (filters.Convert (orig, final))
-						album.Add (item, final);
-					else
-						album.Add (item, orig);
-					
-					if (final != orig)
-						System.IO.File.Delete (final);
+					filters.Convert (req);
+					album.Add (item, req.Current.LocalPath);
+
+//					string orig = item.DefaultVersionUri.LocalPath;
+//
+//					// FIXME the filters need to be able to handle entension
+//					// changes directly themselves.
+//					string final = orig;
+//					if (scale)
+//						final = ImageFile.TempPath (orig, "jpg");
+//					else if (rotate)
+//						final = ImageFile.TempPath (orig);
+//					
+//					if (filters.Convert (orig, final))
+//						album.Add (item, final);
+//					else
+//						album.Add (item, orig);
+//					
+//					if (final != orig)
+//						System.IO.File.Delete (final);
 				}
 
 				progress_dialog.Message = Catalog.GetString ("Done Sending Photos");
