@@ -6,121 +6,26 @@ using GtkSharp;
 using System.Collections;
 using System.IO;
 using System;
-using Mono.Posix;
+using Mono.Unix;
 
 public class ImportCommand : FSpot.GladeDialog {
-	private class PhotoGrid : Table {
-		const int NUM_COLUMNS = 5;
-		const int NUM_ROWS = 4;
-		
-	        const int CELL_WIDTH = 128;
-		const int CELL_HEIGHT = 96;
-
-		const int PADDING = 3;
-
-		Gtk.Image [] image_widgets;
-
-		int position = -1;
-		
-		/*
-		 * This controls whether or wraps as new photos are added.
-		 */
-		public bool Scroll = false;
-
-		public PhotoGrid () : base (NUM_ROWS, NUM_COLUMNS, true)
-		{
-			image_widgets = new Gtk.Image [NUM_ROWS * NUM_COLUMNS];
-
-			int i = 0;
-			for (uint j = 0; j < NUM_ROWS; j++) {
-				for (uint k = 0; k < NUM_COLUMNS; k ++) {
-					Gtk.Image image_widget = new Gtk.Image ();
-
-					image_widget = new Gtk.Image ();
-					image_widget.SetSizeRequest (CELL_WIDTH, CELL_HEIGHT);
-
-					Attach (image_widget, k, k + 1, j, j + 1, 0, 0, PADDING, PADDING);
-
-					image_widgets [i] = image_widget;
-
-					i ++;
-				}
-			}
-		}
-
-		private void ScrollIfNeeded ()
-		{
-			if (position < NUM_COLUMNS * NUM_ROWS)
-				return;
-
-			for (int i = 0; i < NUM_COLUMNS * (NUM_ROWS - 1); i ++) {
-				image_widgets [i].Pixbuf = image_widgets [i + NUM_COLUMNS].Pixbuf;
-			}
-
-			for (int i = NUM_COLUMNS * (NUM_ROWS - 1); i < NUM_COLUMNS * NUM_ROWS; i ++) {
-				// FIXME: Lame, apparently I can't set the Pixbuf property to null.
-				// GTK# bug?
-				image_widgets [i].Pixbuf = new Pixbuf (Colorspace.Rgb, false, 8, 1, 1);
-			}
-
-			position -= NUM_COLUMNS;
-		}
-
-		public void AddThumbnail (Pixbuf thumbnail)
-		{
-			Pixbuf scaled_thumbnail;
-
-			if (thumbnail.Width <= CELL_WIDTH && thumbnail.Height <= CELL_HEIGHT) {
-				scaled_thumbnail = thumbnail;
-			} else {
-				int thumbnail_width, thumbnail_height;
-
-				PixbufUtils.Fit (thumbnail, CELL_WIDTH, CELL_HEIGHT, false,
-						 out thumbnail_width, out thumbnail_height);
-				scaled_thumbnail = thumbnail.ScaleSimple (thumbnail_width, thumbnail_height,
-									  Gdk.InterpType.Bilinear);
-			}
-
-			position = (position + 1) % (NUM_COLUMNS * NUM_ROWS);
-			if (Scroll)
-				ScrollIfNeeded ();
-
-			image_widgets [position].Pixbuf = scaled_thumbnail;
-			if (Scroll)
-				position ++;
-
-
-			if (scaled_thumbnail != thumbnail)
-				scaled_thumbnail.Dispose ();
-			
-		}
-	}
-
 	internal class SourceItem : ImageMenuItem {
 		public ImportSource Source;
 
-		public SourceItem (ImportSource source) : base (source.Name)
+		public SourceItem (ImportSource source) : base (source.Name.Replace ("_", "__"))
 		{
 			this.Source = source;
 	
-			Activated += HandleActivated;
-			
 			Gdk.Pixbuf icon = source.Icon;
 			if (icon != null)
 				this.Image = new Gtk.Image (icon);
 		}
-
-		private void HandleActivated (object sender, EventArgs args)
-		{
-			if (ImportCommand.OpenDialog != null)
-				ImportCommand.OpenDialog.Source = this;
-		}
 	} 
 
 	internal class BrowseSource : ImportSource {
 		public BrowseSource ()
 		{
-			this.Name = Mono.Posix.Catalog.GetString ("Select Folder");
+			this.Name = Catalog.GetString ("Select Folder");
 			this.Icon = PixbufUtils.LoadThemeIcon ("stock_folder", 32);
 		}
 
@@ -161,7 +66,7 @@ public class ImportCommand : FSpot.GladeDialog {
 		public VolumeSource (Gnome.Vfs.Volume vol)
 		{
 			this.Volume = vol;
-			this.Name = vol.DisplayName.Replace ("_", "__");
+			this.Name = vol.DisplayName;
 
 			try {
 				mount_point = new Uri (vol.ActivationUri).LocalPath;
@@ -174,7 +79,7 @@ public class ImportCommand : FSpot.GladeDialog {
                         if (this.Icon == null)
 				this.Icon = PixbufUtils.LoadThemeIcon (vol.Icon, 32);
 			
-			if (this.IsiPodPhoto)
+			if (this.IsIPodPhoto)
 				this.Icon = PixbufUtils.LoadThemeIcon ("gnome-dev-ipod", 32);
 
 			if (this.Icon == null && this.IsCamera)
@@ -191,18 +96,22 @@ public class ImportCommand : FSpot.GladeDialog {
 		private bool IsCamera {
 			get {
 				try {
-					return (Directory.Exists (System.IO.Path.Combine (mount_point, "DCIM")));
+					return (Directory.Exists (Path.Combine (mount_point, "DCIM")));
 				} catch {
 					return false;
 				}
 			}
 		}
 
-		private bool IsiPodPhoto {
+		private bool IsIPodPhoto {
 			get {
+				if (Volume.DeviceType != Gnome.Vfs.DeviceType.MusicPlayer 
+				    && Volume.DeviceType != Gnome.Vfs.DeviceType.Apple)
+					return false;
+
 				try {
-					return (Directory.Exists (System.IO.Path.Combine (mount_point, "Photos")) &&
-						Directory.Exists (System.IO.Path.Combine (mount_point, "iPod_Control")));
+					return (Directory.Exists (Path.Combine (mount_point, "Photos")) &&
+						Directory.Exists (Path.Combine (mount_point, "iPod_Control")));
 				} catch {
 					return false;
 				}
@@ -215,7 +124,7 @@ public class ImportCommand : FSpot.GladeDialog {
 		
 		public DriveSource (Gnome.Vfs.Drive drive) 
 		{
-			this.Name = drive.DisplayName.Replace ("_", "__");
+			this.Name = drive.DisplayName;
 			this.Drive = drive;
 
 			if (drive.IsMounted) {
@@ -236,8 +145,11 @@ public class ImportCommand : FSpot.GladeDialog {
 			this.cam = cam;
 			this.CameraIndex = index;
 
-			//this.Name = String.Format ("{0} ({1})", cam.CameraList.GetName (index), cam.CameraList.GetValue (index));
+#if LONG_NAMES
+			this.Name = String.Format ("{0} ({1})", cam.CameraList.GetName (index), cam.CameraList.GetValue (index));
+#else
 			this.Name = String.Format ("{0}", cam.CameraList.GetName (index));
+#endif
 			this.Icon = PixbufUtils.LoadThemeIcon ("gnome-dev-camera", 32);
 			if (this.Icon == null)
 				this.Icon = PixbufUtils.LoadThemeIcon ("gnome-dev-media-cf", 32);
@@ -258,62 +170,72 @@ public class ImportCommand : FSpot.GladeDialog {
 	
 	private class SourceMenu : Gtk.Menu {
 		public int source_count;
+		ImportCommand command;
 
-		public SourceMenu () {
+		private static Gnome.Vfs.VolumeMonitor monitor = Gnome.Vfs.VolumeMonitor.Get ();
+
+		public SourceMenu (ImportCommand command) {
+			this.command = command;
 			source_count = 0;
-			Gnome.Vfs.VolumeMonitor monitor = Gnome.Vfs.VolumeMonitor.Get ();
 			
-			this.Append (new SourceItem (new BrowseSource ()));
-
+			SourceItem item = new SourceItem (new BrowseSource ());
+			item.Activated += HandleActivated;
+			this.Append (item);
 			this.Append (new Gtk.SeparatorMenuItem ());
 
+			// Add external hard drives to the menu
 			foreach (Gnome.Vfs.Volume vol in monitor.MountedVolumes) {
-				System.Console.WriteLine ("{0} - {1} - {2} {3} {4} {5} {6}",
+				 if (!vol.IsUserVisible || vol.DeviceType == Gnome.Vfs.DeviceType.Unknown)
+					 continue;
+				
+				 System.Console.WriteLine ("{0} - {1} - {2} {3} {4} {5} {6}",
 							  vol.DisplayName, 
-							   vol.Icon, 
+							  vol.Icon, 
 							  vol.VolumeType.ToString (), 
 							  vol.ActivationUri, 
 							  vol.IsUserVisible,
 							  vol.IsMounted,
 							  vol.DeviceType);
-				
+				 
 				 if (vol.Drive != null)
 					 System.Console.WriteLine (vol.Drive.DeviceType.ToString ());
-				 
+
 				 ImportSource source = new VolumeSource (vol);
-#if true
-				 SourceItem item = new SourceItem (source);
-				 if (!vol.IsUserVisible || vol.DeviceType == Gnome.Vfs.DeviceType.Unknown) {
-					 item.Sensitive = false;
-					 continue;
-				 }
+				 item = new SourceItem (source);
+				 item.Activated += HandleActivated;
 				 this.Append (item);
 				 source_count++;
-#else
-				 
-				 this.Append (new SourceItem (source));
-#endif
+
 			}
 
 
 			GPhotoCamera cam = new GPhotoCamera ();
 			cam.DetectCameras ();
-			
-			if (cam.CameraList.Count () > 0)
-				this.Append (new Gtk.SeparatorMenuItem ());
-			
-			source_count += cam.CameraList.Count ();
-			for (int i = 0; i < cam.CameraList.Count (); i++) {
-				if (source_count == 1 || cam.CameraList.GetValue (i) != "usb:") {
-					ImportSource source = new CameraSource (cam, i);
-					this.Append (new SourceItem (source));
-				}
-			}
+			int camera_count = cam.CameraList.Count ();
 
-			if (source_count == 0) {
-				ImportSource source = new BrowseSource (Mono.Posix.Catalog.GetString ("(No Cameras Detected)"),
+			if (camera_count > 0) {
+				source_count += camera_count;
+				for (int i = 0; i < camera_count; i++) {
+					string handle = cam.CameraList.GetValue (i);
+					if (camera_count == 1 || handle != "usb:") {
+						if (handle.StartsWith ("disk:")) {
+							string path = handle.Substring ("disk:".Length);
+
+							if (FindItemPosition (path) != -1)
+								continue;
+						}
+			
+						ImportSource source = new CameraSource (cam, i);
+						item = new SourceItem (source);
+						item.Activated += HandleActivated;
+						this.Append (item);
+					}
+				}
+			} else {
+				ImportSource source = new BrowseSource (Catalog.GetString ("(No Cameras Detected)"),
 									"emblem-camera");
-			SourceItem item = new SourceItem (source);
+				item = new SourceItem (source);
+				item.Activated += HandleActivated;
 				item.Sensitive = false;
 				this.Append (item);
 			}
@@ -332,6 +254,11 @@ public class ImportCommand : FSpot.GladeDialog {
 			this.ShowAll ();
 		}
 
+		private void HandleActivated (object sender, EventArgs args)
+		{
+			command.Source = (SourceItem) sender;
+		}
+
 		public int SourceCount {
 			get {
 				return source_count;
@@ -379,28 +306,26 @@ public class ImportCommand : FSpot.GladeDialog {
 	
 	Tag tag_selected;
 
-	PhotoGrid grid;
 	Gtk.Window main_window;
-	string import_path;
 	FSpot.PhotoList collection;
-	bool cancelled;
 	bool copy;
 	SourceMenu menu;
 
 	int total;
 	PhotoStore store;
-
 	FSpot.Delay step;
 	
 	FSpot.PhotoImageView photo_view;
-	IconView tray;
 	ImportBackend importer;
+	IconView tray;
+
+	FSpot.Delay idle_start; 
 
 	string loading_string;
 
-	private static ImportCommand import_command;
-	protected static ImportCommand OpenDialog {
-		get { return import_command; }
+	string import_path;
+	public string ImportPath {
+		get { return import_path; }
 	}
 	
 	private SourceItem Source {
@@ -440,16 +365,16 @@ public class ImportCommand : FSpot.GladeDialog {
 				MainWindow.Toplevel.ImportCamera (port);
 			}
 
-			Start ();
+			idle_start.Start ();
 		}
 	}
 
 	public ImportCommand (Gtk.Window mw)
 	{
 		main_window = mw;
-		step = new FSpot.Delay (10, new GLib.IdleHandler (Step));
-		loading_string = Mono.Posix.Catalog.GetString ("Loading {0} of {1}");
-		import_command = this;
+		step = new FSpot.Delay (new GLib.IdleHandler (Step));
+		idle_start = new FSpot.Delay (new IdleHandler (Start));
+		loading_string = Catalog.GetString ("Loading {0} of {1}");
 	}
 
 	private void HandleDialogResponse (object obj, ResponseArgs args)
@@ -487,25 +412,36 @@ public class ImportCommand : FSpot.GladeDialog {
 		if (importer == null)
 			return false;
 		
-		ongoing = importer.Step (out photo, out thumbnail, out count);
-		
-		if (thumbnail == null) {
+		try {
+			// FIXME this is really just an incredibly ugly way of dealing
+			// with the recursive DoImport loops we sometimes get into
+			ongoing = importer.Step (out photo, out thumbnail, out count);
+		} catch (ImportException e){
+			System.Console.WriteLine (e);
+			return false;
+		}
+
+		if (photo == null || thumbnail == null) {
 			Console.WriteLine ("Could not import file");
 		} else {
 			//icon_scrolled.Visible = true;
 			collection.Add (photo);
 		
 			//grid.AddThumbnail (thumbnail);
-			if (count < total)
-				UpdateProgressBar (count + 1, total);
 
 			thumbnail.Dispose ();
 		}
+		if (count < total)
+			UpdateProgressBar (count + 1, total);
 
 		if (ongoing && total > 0)
 			return true;
 		else {
 			System.Console.WriteLine ("Stopping");
+			if (progress_bar != null)
+				progress_bar.Text = Catalog.GetString ("Done Loading");
+			
+			AllowFinish = true;
 			return false;
 		}
 	}
@@ -534,9 +470,6 @@ public class ImportCommand : FSpot.GladeDialog {
 		collection.Clear ();
 		collection.Capacity = total;
 
-		cancelled = false;
-		FSpot.ThumbnailGenerator.Default.PushBlock ();
-
 		while (total > 0 && this.Step ()) {
 			System.DateTime start_time = System.DateTime.Now;
 			System.TimeSpan span = start_time - start_time;
@@ -547,28 +480,20 @@ public class ImportCommand : FSpot.GladeDialog {
 			}
 		}
 
-		FSpot.ThumbnailGenerator.Default.PopBlock ();
+		return total;
+	}
+	
+	public void Finish ()
+	{
+		if (idle_start.IsPending || step.IsPending) {
+			AllowFinish = false;
+			return;
+		}
 		
 		if (importer != null)
 			importer.Finish ();
 		
 		importer = null;
-
-		//ThumbnailGenerator.Default.PopBlock ();
-		if (cancelled)
-			return 0;
-		else {
-			if (progress_bar != null)
-				progress_bar.Text = Mono.Posix.Catalog.GetString ("Done Loading");
-			AllowFinish = true;
-			return total;
-		}
-	}
-	
-	public string ImportPath {
-		get {
-			return import_path;
-		}
 	}
 	
 	public void HandleTagToggled (object o, EventArgs args) 
@@ -597,7 +522,7 @@ public class ImportCommand : FSpot.GladeDialog {
 		if (ImportPath != null)
 			file_selector.Filename = ImportPath;
 		else
-			file_selector.Filename = System.Environment.GetEnvironmentVariable ("HOME");
+			file_selector.Filename = FSpot.Global.HomeDirectory;
 
 		int response = file_selector.Run ();
 
@@ -619,16 +544,14 @@ public class ImportCommand : FSpot.GladeDialog {
 		tag_selected = t;
 		//tag_image.Pixbuf = t.Icon;
 		//tag_label.Text = t.Name;
-	
 	}
 
-
 	private void HandleRecurseToggled (object sender, System.EventArgs args)
 	{
 		this.Cancel ();
-		while (Application.EventsPending ())
-			Application.RunIteration ();
-		this.Start ();
+		this.Dialog.Sensitive = false;
+	       
+		idle_start.Start ();
 	}
 
 	public int ImportFromFile (PhotoStore store, string path)
@@ -650,27 +573,28 @@ public class ImportCommand : FSpot.GladeDialog {
 		recurse_check.Toggled += HandleRecurseToggled;
 		copy_check.Toggled += HandleRecurseToggled;
 
-		menu = new SourceMenu ();
+		menu = new SourceMenu (this);
 		source_option_menu.Menu = menu;
 
 		collection = new FSpot.PhotoList (new Photo [0]);
 		tray = new FSpot.ScalingIconView (collection);
 		tray.Selection.Changed += HandleTraySelectionChanged;
-		icon_scrolled.SetSizeRequest (200, 480);
+		icon_scrolled.SetSizeRequest (200, 200);
 		icon_scrolled.Add (tray);
 		//icon_scrolled.Visible = false;
+		tray.DisplayTags = false;
 		tray.Show ();
 
 		photo_view = new FSpot.PhotoImageView (collection);
 		photo_scrolled.Add (photo_view);
-		photo_scrolled.SetSizeRequest (200, 480);
+		photo_scrolled.SetSizeRequest (200, 200);
 		photo_view.Show ();
 
 		//FSpot.Global.ModifyColors (frame_eventbox);
 		FSpot.Global.ModifyColors (photo_scrolled);
 		FSpot.Global.ModifyColors (photo_view);
 
-		photo_view.Pixbuf = PixbufUtils.LoadFromAssembly ("f-spot-logo.png");
+		photo_view.Pixbuf = PixbufUtils.LoadFromAssembly ("f-spot-48.png");
 		photo_view.Fit = true;
 			
 		tag_selected = null;
@@ -687,27 +611,35 @@ public class ImportCommand : FSpot.GladeDialog {
 
 			if (i > 0) {
 				source_option_menu.SetHistory ((uint)i);
-			} else if (System.IO.Directory.Exists (path)) {
+			} else if (Directory.Exists (path)) {
 				SourceItem path_item = new SourceItem (new VfsSource (path));
 				menu.Prepend (path_item);
 				path_item.ShowAll ();
 				SetImportPath (path);
 				source_option_menu.SetHistory (0);
 			} 
+			idle_start.Start ();
 		}
 						
 		ResponseType response = (ResponseType) this.Dialog.Run ();
 		
 		while (response == ResponseType.Ok) {
-			if (System.IO.Directory.Exists (this.ImportPath))
-			    break;
+			try {
+				if (Directory.Exists (this.ImportPath))
+					break;
+			} catch (System.Exception e){
+				System.Console.WriteLine (e);
+				break;
+			}
 
 			HigMessageDialog md = new HigMessageDialog (this.Dialog,
-								    DialogFlags.DestroyWithParent,
-								    MessageType.Error,
-								    ButtonsType.Ok,
-								    Mono.Posix.Catalog.GetString ("Directory does not exist."),
-									    String.Format (Mono.Posix.Catalog.GetString ("The directory you selected \"{0}\" does not exist.  Please choose a different directory"), this.ImportPath));
+			        DialogFlags.DestroyWithParent,
+				MessageType.Error,
+				ButtonsType.Ok,
+				Catalog.GetString ("Directory does not exist."),
+				String.Format (Catalog.GetString ("The directory you selected \"{0}\" does not exist.  " + 
+								  "Please choose a different directory"), this.ImportPath));
+
 			md.Run ();
 			md.Destroy ();
 
@@ -715,6 +647,8 @@ public class ImportCommand : FSpot.GladeDialog {
 		}
 
 		if (response == ResponseType.Ok) {
+			this.Finish ();
+
 			if (attach_check.Active && tag_selected != null) {
 				for (int i = 0; i < collection.Count; i++) {
 					Photo p = collection [i] as Photo;
@@ -739,30 +673,33 @@ public class ImportCommand : FSpot.GladeDialog {
 	private void CreateTagMenu ()
 	{
 		TagMenu tagmenu = new TagMenu (null, MainWindow.Toplevel.Database.Tags);
+		tagmenu.NewTagHandler = HandleNewTagSelected;
 
 		tagmenu.Append (new MenuItem (Catalog.GetString ("Select Tag")));
 		
 		tagmenu.Populate (true);
 		
-		GtkUtil.MakeMenuSeparator (tagmenu);
-		GtkUtil.MakeMenuItem (tagmenu, Catalog.GetString ("Create New Tag"),
-				"f-spot-new-tag", MainWindow.Toplevel.HandleCreateNewCategoryCommand, true);
-		
 		tagmenu.TagSelected += HandleTagMenuSelected;
-		MainWindow.Toplevel.Database.Tags.ItemsAdded += HandleTagsAdded;
 		
 		tagmenu.ShowAll ();
 		tag_option_menu.Menu = tagmenu;
 	}
 
-	private void HandleTagsAdded (object sender, DbItemEventArgs args)
+	private void HandleNewTagSelected (object sender, EventArgs args)
 	{
-		// The user added a tag, so recreate the tag menu
-		CreateTagMenu ();
+		Tag new_tag = MainWindow.Toplevel.CreateTag (Dialog, null);
+
+		if (new_tag != null) {
+			CreateTagMenu ();
+			tag_option_menu.SetHistory ((uint) (tag_option_menu.Menu as TagMenu).GetPosition (new_tag));
+			tag_selected = new_tag;
+		}
 	}
 
 	public void Cancel ()
 	{
+		idle_start.Stop ();
+		step.Stop ();
 		if (importer != null) {
 			importer.Cancel ();
 			importer = null;
@@ -778,27 +715,39 @@ public class ImportCommand : FSpot.GladeDialog {
 		store.Remove (photos);
 	}
 
-	public int Start ()
+	public bool Start ()
 	{
+		if (Dialog != null)
+			Dialog.Sensitive = true;
+
 		if (import_path == null)
-			return 0;
+			return false;
 
 		string [] pathimport =  {ImportPath};
 		//this.Dialog.Destroy();
 		
+		if (copy_check != null)
+			copy = copy_check.Active;
+		
 		bool recurse = true;
 		if (recurse_check != null)
 			recurse = recurse_check.Active;
 		
-		if (collection == null)
-			return 0;
+		importer = new FileImportBackend (store, pathimport, copy, recurse, null);
+		AllowFinish = false;
+		
+		total = importer.Prepare ();
+		
+		if (total > 0)
+			UpdateProgressBar (1, total);
+		
+		collection.Clear ();
+		collection.Capacity = total;
 
-		return DoImport (new FileImportBackend (store, pathimport, copy, recurse, null));
-	}
-
-	public int ImportFromPaths (PhotoStore store, string [] paths)
-	{
-		return ImportFromPaths (store, paths, null);
+		if (total > 0)
+			step.Start ();
+	       
+		return false;
 	}
 
 	public int ImportFromPaths (PhotoStore store, string [] paths, bool copy)
@@ -814,7 +763,11 @@ public class ImportCommand : FSpot.GladeDialog {
 	public int ImportFromPaths (PhotoStore store, string [] paths, Tag [] tags, bool copy)
 	{
 		collection = new FSpot.PhotoList (new Photo [0]);
-		return DoImport (new FileImportBackend (store, paths, copy, true, tags));
+		int count = DoImport (new FileImportBackend (store, paths, copy, true, tags));
+
+		Finish ();
+
+		return count;
 	}
 	
 #if TEST_IMPORT_COMMAND
