@@ -3,14 +3,17 @@ using System;
 using System.IO;
 using FSpot;
 using SemWeb;
+using Mono.Unix;
+
 
 // FIXME TODO: We want to use something like EClippedLabel here throughout so it handles small sizes
 // gracefully using ellipsis.
 
 public class InfoBox : VBox {
 	Delay update_delay;
-	private Photo photo;
-	public Photo Photo {
+
+	private IBrowsableItem photo;
+	public IBrowsableItem Photo {
 		set {
 			photo = value;
 			update_delay.Start ();
@@ -69,20 +72,20 @@ public class InfoBox : VBox {
 	{
 		Table table = new Table (5, 2, false);
 
-		table.Attach (CreateRightAlignedLabel (Mono.Posix.Catalog.GetString ("Name:")), 0, 1, 0, 1,
+		table.Attach (CreateRightAlignedLabel (Catalog.GetString ("Name:")), 0, 1, 0, 1,
 			      AttachOptions.Fill, AttachOptions.Fill, 3, 3);
-		table.Attach (CreateRightAlignedLabel (Mono.Posix.Catalog.GetString ("Version:")), 0, 1, 1, 2,
+		table.Attach (CreateRightAlignedLabel (Catalog.GetString ("Version:")), 0, 1, 1, 2,
 			      AttachOptions.Fill, AttachOptions.Fill, 3, 3);
-		table.Attach (CreateRightAlignedLabel (Mono.Posix.Catalog.GetString ("Date:")), 0, 1, 2, 3,
+		table.Attach (CreateRightAlignedLabel (Catalog.GetString ("Date:")), 0, 1, 2, 3,
 			      AttachOptions.Fill, AttachOptions.Fill, 3, 3);
-		table.Attach (CreateRightAlignedLabel (Mono.Posix.Catalog.GetString ("Size:")), 0, 1, 3, 4,
+		table.Attach (CreateRightAlignedLabel (Catalog.GetString ("Size:")), 0, 1, 3, 4,
 			      AttachOptions.Fill, AttachOptions.Fill, 3, 3);
-		table.Attach (CreateRightAlignedLabel (Mono.Posix.Catalog.GetString ("Exposure:")), 0, 1, 4, 5,
+		table.Attach (CreateRightAlignedLabel (Catalog.GetString ("Exposure:")), 0, 1, 4, 5,
 			      AttachOptions.Fill, AttachOptions.Fill, 3, 3);
 
 		name_entry = new Entry ();
 		name_entry.WidthChars = 1;
-		name_entry.Editable = false;
+		name_entry.IsEditable = false;
 		table.Attach (name_entry, 1, 2, 0, 1,
 			      AttachOptions.Expand | AttachOptions.Fill, AttachOptions.Fill,
 			      3, 0);
@@ -94,6 +97,9 @@ public class InfoBox : VBox {
 		version_option_menu = new OptionMenu ();
 		table.Attach (version_option_menu, 1, 2, 1, 2, AttachOptions.Fill, AttachOptions.Fill, 3, 3);
 
+		date_label.Text = "\n";
+		exposure_info_label.Text = "\n";
+
 		table.ShowAll ();
 
 		Add (table);
@@ -107,9 +113,9 @@ public class InfoBox : VBox {
 		version_option_menu.Menu = new Menu ();	// GTK doesn't like NULL here although that's what we want.
 
 		name_entry.Text = "";
-		date_label.Text = "";
+		date_label.Text = "\n";
 		size_label.Text = "";
-		exposure_info_label.Text = "";
+		exposure_info_label.Text = "\n";
 	}
 
 	private class ImageInfo : StatementSink {
@@ -153,7 +159,7 @@ public class InfoBox : VBox {
 				int real_width;
 				int real_height;
 
-				JpegUtils.GetSize (img.Path, out real_width, out real_height);
+				JpegUtils.GetSize (img.Uri.LocalPath, out real_width, out real_height);
 				width = real_width.ToString ();
 				height = real_height.ToString ();
 			}
@@ -213,7 +219,7 @@ public class InfoBox : VBox {
 					info += "\nISO " + iso_speed;
 				
 				if (info == "")
-					return Mono.Posix.Catalog.GetString ("(None)");
+					return Catalog.GetString ("(None)");
 				
 				return info;
 			}
@@ -224,7 +230,7 @@ public class InfoBox : VBox {
 				if (width != null && height != null)
 					return String.Format ("{0}x{1}", width, height);
 				else 
-					return Mono.Posix.Catalog.GetString ("(Unknown)");
+					return Catalog.GetString ("(Unknown)");
 			}
 		}
 #if USE_EXIF_DATE
@@ -233,7 +239,7 @@ public class InfoBox : VBox {
 				if (date > DateTime.MinValue && date < DateTime.MaxValue)
 					return date.ToShortDateString () + "\n" + date.ToShortTimeString ();
 				else 
-					return Mono.Posix.Catalog.GetString ("(Unknown)");
+					return Catalog.GetString ("(Unknown)");
 			}
 		}
 #endif
@@ -250,11 +256,10 @@ public class InfoBox : VBox {
 			return false;
 		}
 		
-		string path = photo.DefaultVersionUri.LocalPath;
-		name_entry.Text = System.IO.Path.GetFileName (path);
+		name_entry.Text = photo.Name;
 		try {
 			//using (new Timer ("building info")) {
-				img = ImageFile.Create (path);
+				img = ImageFile.Create (photo.DefaultVersionUri);
 				info = new ImageInfo (img);
 				//}
 		} catch (System.Exception e) {
@@ -270,25 +275,36 @@ public class InfoBox : VBox {
 		date_label.Text = info.Date;
 #else
 		date_label.Text = String.Format ("{0}\n{1}",
-						 photo.Time.ToShortDateString (),
-						 photo.Time.ToShortTimeString ());
+						 photo.Time.ToLocalTime ().ToShortDateString (),
+						 photo.Time.ToLocalTime ().ToShortTimeString ());
 #endif
-		version_option_menu.Sensitive = true;
-		PhotoVersionMenu menu = new PhotoVersionMenu (photo);
-		menu.VersionIdChanged += new PhotoVersionMenu.VersionIdChangedHandler (HandleVersionIdChanged);
-		menu.WidthRequest = version_option_menu.Allocation.Width;
-		version_option_menu.Menu = menu;
+		
 
-		uint i = 0;
-		foreach (uint version_id in photo.VersionIds) {
-			if (version_id == photo.DefaultVersionId) {
-				// FIXME GTK# why not just .History = i ?
-				version_option_menu.SetHistory (i);
-				break;
+		Photo p = photo as Photo;
+		if (p != null) {
+			version_option_menu.Visible = true;
+			version_option_menu.Sensitive = true;
+			PhotoVersionMenu menu = new PhotoVersionMenu (p);
+			menu.VersionIdChanged += new PhotoVersionMenu.VersionIdChangedHandler (HandleVersionIdChanged);
+			menu.WidthRequest = version_option_menu.Allocation.Width;
+			version_option_menu.Menu = menu;
+			
+			uint i = 0;
+			foreach (uint version_id in p.VersionIds) {
+				if (version_id == p.DefaultVersionId) {
+					// FIXME GTK# why not just .History = i ?
+					version_option_menu.SetHistory (i);
+					break;
+				}
+				i++;
 			}
-			i++;
+		} else {
+			version_option_menu.Visible = false;
+			version_option_menu.Sensitive = false;
+			version_option_menu.Menu = null;
 		}
 
+
 		return false;
 	}
 
