@@ -2,6 +2,8 @@ using Gtk;
 using System;
 using System.Runtime.InteropServices;
 using Cairo;
+using Mono.Unix;
+using FSpot.Widgets;
 
 namespace FSpot {
 	public class Sharpener : Loupe {
@@ -43,18 +45,22 @@ namespace FSpot {
 			
 			try {
 				Gdk.Pixbuf orig = view.Pixbuf;
-				Gdk.Pixbuf final = PixbufUtils.UnsharpMask (orig, radius_spin.Value, amount_spin.Value, threshold_spin.Value);
+				Gdk.Pixbuf final = PixbufUtils.UnsharpMask (orig,
+									    radius_spin.Value,
+									    amount_spin.Value,
+									    threshold_spin.Value);
 				
 				bool create_version = photo.DefaultVersionId == Photo.OriginalVersionId;
 
 				photo.SaveVersion (final, create_version);
 			} catch (System.Exception e) {
-				string msg = Mono.Posix.Catalog.GetString ("Error saving sharpened photo");
-				string desc = String.Format (Mono.Posix.Catalog.GetString ("Received exception \"{0}\". Unable to save photo {1}"),
+				string msg = Catalog.GetString ("Error saving sharpened photo");
+				string desc = String.Format (Catalog.GetString ("Received exception \"{0}\". Unable to save photo {1}"),
 							     e.Message, photo.Name);
 				
 				HigMessageDialog md = new HigMessageDialog (this, DialogFlags.DestroyWithParent, 
-									    Gtk.MessageType.Error, ButtonsType.Ok, 
+									    Gtk.MessageType.Error,
+									    ButtonsType.Ok, 
 									    msg,
 									    desc);
 				md.Run ();
@@ -78,7 +84,7 @@ namespace FSpot {
 		{
 			base.BuildUI ();
 
-			string title = Mono.Posix.Catalog.GetString ("Sharpen");
+			string title = Catalog.GetString ("Sharpen");
 			dialog = new Gtk.Dialog (title, (Gtk.Window) this,
 						 DialogFlags.DestroyWithParent, new object [0]);
 			dialog.BorderWidth = 12;
@@ -88,9 +94,9 @@ namespace FSpot {
 			table.ColumnSpacing = 6;
 			table.RowSpacing = 6;
 			
-			table.Attach (SetFancyStyle (new Gtk.Label (Mono.Posix.Catalog.GetString ("Amount:"))), 0, 1, 0, 1);
-			table.Attach (SetFancyStyle (new Gtk.Label (Mono.Posix.Catalog.GetString ("Radius:"))), 0, 1, 1, 2);
-			table.Attach (SetFancyStyle (new Gtk.Label (Mono.Posix.Catalog.GetString ("Threshold:"))), 0, 1, 2, 3);
+			table.Attach (SetFancyStyle (new Gtk.Label (Catalog.GetString ("Amount:"))), 0, 1, 0, 1);
+			table.Attach (SetFancyStyle (new Gtk.Label (Catalog.GetString ("Radius:"))), 0, 1, 1, 2);
+			table.Attach (SetFancyStyle (new Gtk.Label (Catalog.GetString ("Threshold:"))), 0, 1, 2, 3);
 			
 			SetFancyStyle (amount_spin = new Gtk.SpinButton (0.00, 100.0, .01));
 			SetFancyStyle (radius_spin = new Gtk.SpinButton (1.0, 50.0, .01));
@@ -121,7 +127,6 @@ namespace FSpot {
 			dialog.VBox.PackStart (table);
 			dialog.ShowAll ();
 		}
-
 	}
 
 	public class Loupe : Gtk.Window {
@@ -132,7 +137,7 @@ namespace FSpot {
 		protected Gdk.Pixbuf overlay;
 		private int radius = 128;
 		private int inner = 128;
-		private int border = 5;
+		private int border = 6;
 		private double angle = Math.PI / 4;
 		Gdk.Point start;
 		Gdk.Point start_hot;
@@ -143,16 +148,38 @@ namespace FSpot {
 		{ 
 			this.view = view;
 			Decorated = false;
+			
+			Gtk.Window win = (Gtk.Window) view.Toplevel;
 
-			Gdk.Visual visual = Gdk.Visual.GetBestWithDepth (32);
-			if (visual != null)
-				Colormap = new Gdk.Colormap (visual, false);
-			else
-				use_shape_ext = true;
+			win.GetPosition (out old_win_pos.X, out old_win_pos.Y);
+			win.ConfigureEvent += HandleToplevelConfigure;
+
+			TransientFor = win;
+			DestroyWithParent = true;
 
 			BuildUI ();
 		}
 
+		Gdk.Point old_win_pos;
+		[GLib.ConnectBefore]
+		public void HandleToplevelConfigure (object o, ConfigureEventArgs args)
+		{
+			int x, y;
+			int loupe_x, loupe_y;
+
+			x = args.Event.X - old_win_pos.X;
+			y = args.Event.Y - old_win_pos.Y;
+			
+			GetPosition (out loupe_x, out loupe_y);
+			Move (loupe_x + x, loupe_y + y);
+
+			old_win_pos.X = args.Event.X;
+			old_win_pos.Y = args.Event.Y;
+		}
+		
+		// FIXME
+		//screen "composited-changed"
+
 		public int Radius {
 			get {
 				return radius;
@@ -182,41 +209,34 @@ namespace FSpot {
 				return angle;
 			}
 			set {
-				Gdk.Point then = GetHotspot ();
+				Gdk.Point then = hotspot;
 				angle = value;
-				Gdk.Point now = GetHotspot ();
-				System.Console.WriteLine ("{0} now {1}", then, now);
+				Layout ();
+				Gdk.Point now = hotspot;
+				//System.Console.WriteLine ("{0} now {1}", then, now);
 				int x, y;
 				GdkWindow.GetOrigin (out x, out y);
+				//GdkWindow.MoveResize (x + then.X - now.X, y + then.Y - now.Y, Bounds.Width, Bounds.Height);
+				ShapeWindow ();
 				Move (x + then.X - now.X, y + then.Y - now.Y);
-				QueueResize ();
+				//QueueResize ();
 			}
 		}
 
 		protected override void OnSizeAllocated (Gdk.Rectangle allocation)
 		{
-			base.OnSizeAllocated (allocation);
-			if (use_shape_ext) {
-				try {
-					Gdk.Pixmap bitmap = new Gdk.Pixmap (GdkWindow, 
-									    allocation.Width, 
-									    allocation.Height, 1);
-					Graphics g = CreateDrawable (bitmap);
-					DrawShape (g, allocation.Width, allocation.Height);
-					((IDisposable)g).Dispose ();
-					ShapeCombineMask (bitmap, 0, 0);
-					bitmap.Dispose ();
-				} catch (System.Exception e) {
-					use_shape_ext = false;
-				}
-			} 
+			if (IsRealized)
+				ShapeWindow ();
 
-			if (! use_shape_ext) {
-				Realize ();
-				Graphics g = CreateDrawable (GdkWindow);
-				DrawShape (g, Allocation.Width, Allocation.Height);
-				((IDisposable)g).Dispose ();
-			}
+			base.OnSizeAllocated (allocation);
+		}
+
+		protected override void OnRealized ()
+		{
+			use_shape_ext = ! (CompositeUtils.IsComposited (Screen) && CompositeUtils.SetRgbaColormap (this));
+			
+			base.OnRealized ();
+			ShapeWindow ();
 		}
 
 		public void SetSamplePoint (Gdk.Point p)
@@ -250,7 +270,7 @@ namespace FSpot {
 			int small = (int) (radius * view.Zoom);
 			if (small != inner) {
 				inner = small;
-				this.QueueResize ();
+				QueueResize ();
 			}
 
 			source = new Gdk.Pixbuf (view.Pixbuf,
@@ -274,69 +294,97 @@ namespace FSpot {
 			SetSamplePoint (view.WindowCoordsToImage (coords));
 		}
 
-		private Gdk.Point GetHotspot ()
+		private void ShapeWindow ()
 		{
-			Matrix m = new Matrix ();
-			m.InitIdentity ();
+			Layout ();
+			Gdk.Pixmap bitmap = new Gdk.Pixmap (GdkWindow, 
+							    Allocation.Width, 
+							    Allocation.Height, 1);
+			
+			Context g = CairoUtils.CreateContext (bitmap);
+			DrawShape (g, Allocation.Width, Allocation.Height);
+			((IDisposable)g).Dispose ();
 
-			int inner_x = radius + border + inner;
-			int cx = radius + 2 * border;
-			int cy = radius + 2 * border;
-
-			m.Translate (cx, cy);
-			m.Rotate (angle);
-			
-			double hx = inner_x;
-			double hy = 0;
-			
-			m.TransformPoint (ref hx, ref hy);
-			
-			Gdk.Point p;
-			p.X = (int)hx;
-			p.Y = (int)hy;
-			
-			return p;
+			if (use_shape_ext)
+				ShapeCombineMask (bitmap, 0, 0);
+			else {
+				Context rgba = CairoUtils.CreateContext (GdkWindow);
+				DrawShape (rgba, Allocation.Width, Allocation.Height);
+				((IDisposable)rgba).Dispose ();
+				try {
+					CompositeUtils.InputShapeCombineMask (this, bitmap, 0,0);
+				} catch (EntryPointNotFoundException) {
+					System.Console.WriteLine ("Warning: gtk+ version doesn't support input shapping");
+				}
+			}
+			bitmap.Dispose ();
 		}
 		
-		private void DrawShape (Cairo.Graphics g, int width, int height)
+		Gdk.Point Center;
+	        Requisition Bounds;
+		
+		public void Layout ()
+		{
+			double a = radius + border;
+			double b = inner + border;
+			double x_proj = (a + b - border) * Math.Cos (angle);
+			double y_proj = (a + b - border) * Math.Sin (angle);
+			
+			Center.X = (int) Math.Ceiling (Math.Max (-x_proj + b, a));
+			Center.Y = (int) Math.Ceiling (Math.Max (-y_proj + b, a));
+
+			Bounds.Width = (int) Math.Ceiling (Math.Max (Math.Abs (x_proj) + b, a) + b + a);
+			Bounds.Height = (int) Math.Ceiling (Math.Max (Math.Abs (y_proj) + b, a) + b + a);
+			
+			hotspot.X = (int) Math.Ceiling (Center.X + x_proj);
+			hotspot.Y = (int) Math.Ceiling (Center.Y + y_proj);
+		}
+		
+		private void DrawShape (Cairo.Context g, int width, int height)
 		{
 			int inner_x = radius + border + inner;
-			int cx = radius + 2 * border;
-			int cy = radius + 2 * border;
+			int cx = Center.X;
+			int cy = Center.Y;
 			
 			g.Operator = Operator.Source;
-			g.Color = new Cairo.Color (0,0,0,0);
+			g.Source = new SolidPattern (new Cairo.Color (0,0,0,0));
 			g.Rectangle (0, 0, width, height);
 			g.Paint ();
 
 			g.NewPath ();
-			g.Operator = Operator.Over;
 			g.Translate (cx, cy);
 			g.Rotate (angle);
-			g.Color = new Cairo.Color (0.4, 0.4, 0.4, .7);
 			
+			g.Source = new SolidPattern (new Cairo.Color (0.2, 0.2, 0.2, .6));
+			g.Operator = Operator.Over;
 			g.Rectangle (0, - (border + inner), inner_x, 2 * (border + inner));
 			g.Arc (inner_x, 0, inner + border, 0, 2 * Math.PI);
 			g.Arc (0, 0, radius + border, 0, 2 * Math.PI);
 			g.Fill ();
 
-			double hx = inner_x;
-			double hy = 0;
-			
-			UserToDevice (g, ref hx, ref hy);
-			hotspot.X = (int)hx;
-			hotspot.Y = (int)hy;
-
-			g.Color = new Cairo.Color (0, 0, 0, 1.0);
+			g.Source = new SolidPattern (new Cairo.Color (0, 0, 0, 1.0));
 			g.Operator = Operator.DestOut;
 			g.Arc (inner_x, 0, inner, 0, 2 * Math.PI);
+#if true
 			g.Fill ();
+#else
+			g.FillPreserve ();
 
+			g.Operator = Operator.Over;
+			RadialGradient rg = new RadialGradient (inner_x - (inner * 0.3), inner * 0.3 , inner * 0.1, inner_x, 0, inner);
+			rg.AddColorStop (0, new Cairo.Color (0.0, 0.2, .8, 0.5)); 
+			rg.AddColorStop (0.7, new Cairo.Color (0.0, 0.2, .8, 0.1)); 
+			rg.AddColorStop (1.0, new Cairo.Color (0.0, 0.0, 0.0, 0.0));
+			g.Source = rg;
+			g.Fill ();
+			rg.Destroy ();
+#endif
 			g.Operator = Operator.Over;
 			g.Matrix = new Matrix ();
 			g.Translate (cx, cy);
 			if (source != null)
 				SetSourcePixbuf (g, source, -source.Width / 2, -source.Height / 2);
+
 			g.Arc (0, 0, radius, 0, 2 * Math.PI);
 			g.Fill ();
 
@@ -345,14 +393,14 @@ namespace FSpot {
 				g.Arc (0, 0, radius, angle, angle + Math.PI);
 				g.ClosePath ();
 				g.FillPreserve ();
-				g.Color = new Cairo.Color (1.0, 1.0, 1.0, 1.0);
+				g.Source = new SolidPattern (new Cairo.Color (1.0, 1.0, 1.0, 1.0));
 				g.Stroke ();
 			}
 		}
 
 		protected override bool OnExposeEvent (Gdk.EventExpose args)
 		{
-			Graphics g = CreateDrawable (GdkWindow);
+			Context g = CairoUtils.CreateContext (GdkWindow);
 			DrawShape (g, Allocation.Width, Allocation.Height);
 			//base.OnExposeEvent (args);
 			((IDisposable)g).Dispose ();
@@ -372,8 +420,9 @@ namespace FSpot {
 		{
 		        pos.X = (int) args.Event.XRoot - start.X;
 		        pos.Y = (int) args.Event.YRoot - start.Y;
-			root_pos.X = (int) args.Event.XRoot - start_root.X;
-			root_pos.Y = (int) args.Event.YRoot - start_root.Y;
+
+			root_pos.X = (int) args.Event.XRoot;
+			root_pos.Y = (int) args.Event.YRoot;
 
 			if (dragging)
 				drag.Start ();
@@ -387,11 +436,24 @@ namespace FSpot {
 			if (!rotate) {
 				return MoveWindow ();
 			} else {
-				double dist = Math.Sqrt (root_pos.X * root_pos.X + root_pos.Y * root_pos.Y);
-				double angle = Math.Asin (root_pos.X / dist);
-				if (root_pos.Y > 0)
-					angle += Math.PI;
-				System.Console.WriteLine ("angle = {0}", angle);
+				Gdk.Point initial = start_root;
+				Gdk.Point hot = start_hot;
+				Gdk.Point win = Gdk.Point.Zero;
+				
+				hot.X += win.X;
+				hot.Y += win.Y;
+				
+				initial.X -= hot.X;
+				initial.Y -= hot.Y;
+				Gdk.Point now = root_pos;
+				now.X -= hot.X;
+				now.Y -= hot.Y;
+				
+				Vector v1 = new Vector (initial);
+				Vector v2 = new Vector (now);
+
+				double angle = Vector.AngleBetween (v1, v2);
+				
 				Angle = start_angle + angle;
 				return false;	
 			}
@@ -434,6 +496,13 @@ namespace FSpot {
 				if (args.Event.Button == 1) {
 					start = new Gdk.Point ((int)args.Event.X, (int)args.Event.Y);
 					start_root = new Gdk.Point ((int)args.Event.XRoot, (int)args.Event.YRoot);
+					start_hot = hotspot;
+
+					Gdk.Point win;
+					GdkWindow.GetOrigin (out win.X, out win.Y);
+					start_hot.X += win.X;
+					start_hot.Y += win.Y;
+
 					dragging = true;
 					rotate = (args.Event.State & Gdk.ModifierType.ShiftMask) > 0;
 					start_angle = Angle;
@@ -478,9 +547,9 @@ namespace FSpot {
                 [DllImport ("libcairo-2.dll")]
                 static extern void cairo_user_to_device (IntPtr cr, ref double x, ref double y);
 
-		static void UserToDevice (Graphics g, ref double x, ref double y)
+		static void UserToDevice (Context ctx, ref double x, ref double y)
 		{
-			cairo_user_to_device (g.Handle, ref x, ref y);
+			cairo_user_to_device (ctx.Handle, ref x, ref y);
 		}
 
 		[DllImport("libgdk-2.0-0.dll")]
@@ -489,23 +558,18 @@ namespace FSpot {
 								double        pixbuf_x,
 								double        pixbuf_y);
 
-		static void SetSourcePixbuf (Graphics g, Gdk.Pixbuf pixbuf, double x, double y)
+		static void SetSourcePixbuf (Context ctx, Gdk.Pixbuf pixbuf, double x, double y)
 		{
-			gdk_cairo_set_source_pixbuf (g.Handle, pixbuf.Handle, x, y);
+			gdk_cairo_set_source_pixbuf (ctx.Handle, pixbuf.Handle, x, y);
 		}
 
-		[DllImport("libgdk-2.0-0.dll")]
-		static extern IntPtr gdk_cairo_create (IntPtr raw);
-		
-		public static Cairo.Graphics CreateDrawable (Gdk.Drawable drawable)
+		protected override void OnSizeRequested (ref Requisition requisition) 
 		{
-			Cairo.Graphics g = new Cairo.Graphics (gdk_cairo_create (drawable.Handle));
-			if (g == null) 
-				throw new Exception ("Couldn't create Cairo Graphics!");
-			
-			return g;
+			Layout ();
+			requisition = Bounds;
+			base.OnSizeRequested (ref requisition);
 		}
-		
+
 		protected virtual void BuildUI ()
 		{
 			SetFancyStyle (this);
@@ -519,7 +583,6 @@ namespace FSpot {
 			view.ZoomChanged += HandleViewZoomChanged;
 
 			SetSamplePoint (Gdk.Point.Zero);
-			SetSizeRequest (400, 400);
 
 			AddEvents ((int) (Gdk.EventMask.PointerMotionMask
 					  | Gdk.EventMask.ButtonPressMask
