@@ -1,6 +1,6 @@
 using SemWeb;
 using SemWeb.Util;
-using Mono.Posix;
+using Mono.Unix;
 using FSpot.Xmp;
 
 namespace FSpot {
@@ -87,7 +87,6 @@ namespace FSpot {
 		public Description (string predicate, string title, string description, ValueFormat formater)
 		{
 			this.predicate = predicate;
-			this.description = description;
 			this.title = title;
 			this.formater = formater;
 		}
