@@ -1,4 +1,5 @@
 using System;
+using FSpot.Editors;
 
 namespace FSpot {
 	public enum ProgressType {
@@ -16,6 +17,8 @@ namespace FSpot {
 		protected FSpot.Loupe sharpener;
 		ProgressType load_async = ProgressType.Full;
 		bool progressive_display;
+		public GdkGlx.Context Glx;
+		private Editor editor;
 
 		public PhotoImageView (IBrowsableCollection query)
 		{
@@ -25,9 +28,12 @@ namespace FSpot {
 			loader.Done += HandleDone;
 
 			Accelerometer.OrientationChanged += HandleOrientationChanged;
+			HandleRealized (null, null);
 
 			this.SizeAllocated += HandleSizeAllocated;
 			this.KeyPressEvent += HandleKeyPressEvent;
+			//this.Realized += HandleRealized;
+			this.Unrealized += HandleUnrealized;
 			this.ScrollEvent += HandleScrollEvent;
 			this.item = new BrowsablePointer (query, -1);
 			item.Changed += PhotoItemChanged;
@@ -65,6 +71,19 @@ namespace FSpot {
 #endif
 		}
 
+		public Editor Editor {
+			get { return editor; }
+			set {
+				Editor tmp = editor;
+
+				editor = value;
+
+				if (tmp != null)
+					tmp.Destroy ();
+				
+			}
+		}
+
 		public Gdk.Pixbuf CompletePixbuf ()
 		{
 			loader.LoadToDone ();
@@ -84,6 +103,25 @@ namespace FSpot {
 			PhotoItemChanged (Item, null);
 		}
 
+		[GLib.ConnectBefore]
+		private void HandleRealized (object sender, EventArgs args)
+		{
+			int [] attr = new int [] {
+				(int) GdkGlx.GlxAttribute.Rgba,
+				(int) GdkGlx.GlxAttribute.DepthSize, 16,
+				(int) GdkGlx.GlxAttribute.DoubleBuffer,
+				(int) GdkGlx.GlxAttribute.None
+			};
+
+			Glx = new GdkGlx.Context (Screen, attr);
+		        Colormap = Glx.GetColormap ();
+		}
+
+		private void HandleUnrealized (object sender, EventArgs args)
+		{
+			Glx.Destroy ();
+		}
+
 		// Display.
 		private void HandlePixbufAreaUpdated (object sender, AreaUpdatedArgs args)
 		{
@@ -94,6 +132,7 @@ namespace FSpot {
 			this.QueueDrawArea (area.X, area.Y, area.Width, area.Height);
 		}
 		
+
 		private void HandlePixbufPrepared (object sender, AreaPreparedArgs args)
 		{
 			if (!ShowProgress)
@@ -424,9 +463,11 @@ namespace FSpot {
 					loupe = new Loupe (this);
 					loupe.Destroyed += HandleLoupeDestroy;
 				}
-
 				loupe.Show ();
 				break;
+			case Gdk.Key.e:
+				Editor = new FSpot.Editors.SoftFocus (this);
+				break;
 			case Gdk.Key.equal:
 			case Gdk.Key.plus:
 			case Gdk.Key.KP_Add:
