@@ -1,6 +1,6 @@
 using System.Net;
 using System;
-using Mono.Posix;
+using Mono.Unix;
 
 namespace FSpot
 {
@@ -40,6 +40,10 @@ namespace FSpot
 		public const string VIEWER_HEIGHT = "/apps/f-spot/ui/viewer_height";
 		public const string VIEWER_MAXIMIZED = "/apps/f-spot/ui/viewer_maximized";
 		public const string VIEWER_SHOW_TOOLBAR = "/apps/f-spot/ui/viewer_show_toolbar";
+		public const string VIEWER_SHOW_FILENAMES = "/apps/f-spot/ui/viewer_show_filenames";
+ 		public const string VIEWER_INTERPOLATION = "/apps/f-spot/viewer/interpolation";
+ 		public const string VIEWER_TRANS_COLOR = "/apps/f-spot/viewer/trans_color";
+ 		public const string VIEWER_TRANSPARENCY = "/apps/f-spot/viewer/transparency";
 		
 		public const string SHOW_TOOLBAR = "/apps/f-spot/ui/show_toolbar";
 		public const string SHOW_SIDEBAR = "/apps/f-spot/ui/show_sidebar";
@@ -61,11 +65,14 @@ namespace FSpot
 		public const string EXPORT_FLICKR_TAGS = "/apps/f-spot/export/flickr/tags";
 		public const string EXPORT_FLICKR_STRIP_META = "/apps/f-spot/export/flickr/strip_meta";
 		public const string EXPORT_FLICKR_EMAIL = "/apps/f-spot/export/flickr/email";
-		public const string EXPORT_FLICKR_TOKEN = "/apps/f-spot/export/flickr/token";
 		public const string EXPORT_FLICKR_PUBLIC = "/apps/f-spot/export/flickr/public";
 		public const string EXPORT_FLICKR_FRIENDS = "/apps/f-spot/export/flickr/friends";
 		public const string EXPORT_FLICKR_FAMILY = "/apps/f-spot/export/flickr/family";
 
+		public const string EXPORT_TOKEN_FLICKR ="/apps/f-spot/export/tokens/flickr"; 
+		public const string EXPORT_TOKEN_23HQ ="/apps/f-spot/export/tokens/23hq"; 
+		public const string EXPORT_TOKEN_ZOOOMR ="/apps/f-spot/export/tokens/zooomr"; 
+
 		public const string EXPORT_GALLERY_SCALE = "/apps/f-spot/export/gallery/scale";
 		public const string EXPORT_GALLERY_SIZE = "/apps/f-spot/export/gallery/size";
 		public const string EXPORT_GALLERY_BROWSER = "/apps/f-spot/export/gallery/browser";
@@ -77,6 +84,17 @@ namespace FSpot
 		public const string EXPORT_PICASAWEB_ROTATE = "/apps/f-spot/export/picasaweb/rotate";
 		public const string EXPORT_PICASAWEB_BROWSER = "/apps/f-spot/export/picasaweb/browser";
 
+		public const string EXPORT_SMUGMUG_SCALE = "/apps/f-spot/export/smugmug/scale";
+		public const string EXPORT_SMUGMUG_SIZE = "/apps/f-spot/export/smugmug/size";
+		public const string EXPORT_SMUGMUG_ROTATE = "/apps/f-spot/export/smugmug/rotate";
+		public const string EXPORT_SMUGMUG_BROWSER = "/apps/f-spot/export/smugmug/browser";
+
+		public const string EXPORT_FOLDER_SCALE = "/apps/f-spot/export/folder/scale";
+		public const string EXPORT_FOLDER_SIZE = "/apps/f-spot/export/folder/size";
+		public const string EXPORT_FOLDER_OPEN = "/apps/f-spot/export/folder/browser";
+		public const string EXPORT_FOLDER_ROTATE = "/apps/f-spot/export/folder/rotate";
+		public const string EXPORT_FOLDER_METHOD = "/apps/f-spot/export/folder/method";
+		
  		public const string EXPORT_EMAIL_SIZE = "/apps/f-spot/export/email/size";
  		public const string EXPORT_EMAIL_ROTATE = "/apps/f-spot/export/email/auto_rotate";
  		public const string EXPORT_EMAIL_DELETE_TIMEOUT_SEC = "/apps/f-spot/export/email/delete_timeout_seconds";
@@ -87,6 +105,8 @@ namespace FSpot
 
 		public const string METADATA_EMBED_IN_IMAGE = "/apps/f-spot/metadata/embed_in_image";
 
+		public const string EDIT_REDEYE_THRESHOLD = "/apps/f-spot/edit/redeye_threshold";
+
 		public const string GNOME_SCREENSAVER_THEME = "/apps/gnome-screensaver/themes";
 		public const string GNOME_SCREENSAVER_MODE = "/apps/gnome-screensaver/mode";
 
@@ -132,12 +152,19 @@ namespace FSpot
 			case SHOW_TIMELINE:
 			case SHOW_TAGS:
 			case SHOW_DATES:
+			case VIEWER_SHOW_FILENAMES:
 			case EXPORT_PICASAWEB_SCALE:
 			case EXPORT_PICASAWEB_ROTATE:
 			case EXPORT_PICASAWEB_BROWSER:
+			case EXPORT_SMUGMUG_SCALE:
+			case EXPORT_SMUGMUG_ROTATE:
+			case EXPORT_SMUGMUG_BROWSER:
+			case EXPORT_FOLDER_SCALE:
 				return true;
 			
 			case EXPORT_PICASAWEB_SIZE:
+			case EXPORT_SMUGMUG_SIZE:
+			case EXPORT_FOLDER_SIZE:
 				return 800;
 		
 			case SIDEBAR_POSITION:
@@ -151,9 +178,20 @@ namespace FSpot
 			case EXPORT_EMAIL_SIZE:
 				return 3;	// medium size 640px
 			case EXPORT_EMAIL_ROTATE:
-				return true;	// auto rotate original photos
+			case EXPORT_FOLDER_OPEN:
+			case EXPORT_FOLDER_ROTATE:
+			case VIEWER_INTERPOLATION:
+				return true;
 			case EXPORT_EMAIL_DELETE_TIMEOUT_SEC:
 				return 30;	// delete temporary email pictures after 30 seconds
+			case EXPORT_FOLDER_METHOD:
+				return "static";
+			case VIEWER_TRANSPARENCY:
+				return "NONE";
+			case VIEWER_TRANS_COLOR:
+				return "#000000";
+			case EDIT_REDEYE_THRESHOLD:
+				return -15;
 			}
 
 			return null;
@@ -165,7 +203,7 @@ namespace FSpot
 				return Client.Get (key);
 			} catch (GConf.NoSuchKeyException) {
 				if (default_val != null)
-					Client.Set (key, default_val);
+					Set (key, default_val);
 				
 				return default_val;
 			}
@@ -179,7 +217,7 @@ namespace FSpot
 				object default_val = GetDefault (key);
 
 				if (default_val != null)
-					Client.Set (key, default_val);
+					Set (key, default_val);
 
 				return default_val;
 			}
@@ -187,7 +225,21 @@ namespace FSpot
 
 		public static void Set (string key, object value)
 		{
-			Client.Set (key, value);
+			try {
+				Client.Set (key, value);
+			} catch {
+				Console.WriteLine ("Unable to write this gconf key :"+key);
+			}
+		}
+
+		public static void SetAsBackground (string path)
+		{
+			Client.Set ("/desktop/gnome/background/color_shading_type", "solid");
+			Client.Set ("/desktop/gnome/background/primary_color", "#000000");
+			Client.Set ("/desktop/gnome/background/picture_options", "stretched");
+			Client.Set ("/desktop/gnome/background/picture_opacity", 100);
+			Client.Set ("/desktop/gnome/background/picture_filename", path);
+			Client.Set ("/desktop/gnome/background/draw_background", true);
 		}
 
 		public static event GConf.NotifyEventHandler SettingChanged;
@@ -236,7 +288,7 @@ namespace FSpot
 				string password = (string) Preferences.Get (PaswordKey, "");
 
 				proxy.Credentials = new NetworkCredential (username, password);
-			} catch (Exception e) {
+			} catch (Exception) {
 				proxy = null;
 			}
 
