@@ -84,7 +84,8 @@ namespace FSpot {
 
 			collection = new UriCollection (uris);
 
-			TargetEntry [] dest_table = { new TargetEntry ("text/uri-list", 0, 0) };
+			TargetEntry [] dest_table = {   new TargetEntry ("text/uri-list", 0, 0),
+							new TargetEntry ("text/plain", 0, 1)};
 			
 			directory_view = new IconView (collection);
 			directory_view.Selection.Changed += HandleSelectionChanged;
@@ -114,6 +115,8 @@ namespace FSpot {
 		
 			LoadPreference (Preferences.VIEWER_SHOW_TOOLBAR);
  			LoadPreference (Preferences.VIEWER_INTERPOLATION);
+			LoadPreference (Preferences.VIEWER_TRANSPARENCY);
+			LoadPreference (Preferences.VIEWER_TRANS_COLOR);
 
 			ShowSidebar = collection.Count > 1;
 
@@ -132,6 +135,7 @@ namespace FSpot {
 			
 			if (collection.Count > 0)
 				directory_view.Selection.Add (0);
+
 		}
 
 		public void HandleCollectionChanged (IBrowsableCollection collection)
@@ -199,7 +203,12 @@ namespace FSpot {
 
 		void HandleExportToFlickr (object sender, System.EventArgs args)
 		{
-			new FSpot.FlickrExport (new PhotoArray (directory_view.Selection.Items), false);
+			new FSpot.FlickrExport (FlickrNet.SupportedService.Flickr, new PhotoArray (directory_view.Selection.Items), false);
+		}
+
+		void HandleExportTo23hq (object sender, System.EventArgs args)
+		{
+			new FSpot.FlickrExport (FlickrNet.SupportedService.TwentyThreeHQ, new PhotoArray (directory_view.Selection.Items), false);
 		}
 
 		void HandleExportToPicasaweb (object sender, System.EventArgs args)
@@ -207,6 +216,11 @@ namespace FSpot {
 			new FSpot.GoogleExport (new PhotoArray (directory_view.Selection.Items));
 		}
 
+		void HandleExportToSmugMug (object sender, System.EventArgs args)
+		{
+			new FSpot.SmugMugExport (new PhotoArray (directory_view.Selection.Items)); 	
+		}
+
 		void HandleExportToGallery (object sender, System.EventArgs args)
 		{
 			new FSpot.GalleryExport (new PhotoArray (directory_view.Selection.Items)); 
@@ -445,6 +459,7 @@ namespace FSpot {
 		
 		switch (args.Info) {
 		case 0:
+		case 1:
 			/* 
 			 * If the drop is coming from inside f-spot then we don't want to import 
 			 */
@@ -465,7 +480,7 @@ namespace FSpot {
 			IBrowsableItem item = image_view.Item.Current;
 			System.Text.StringBuilder sb = new System.Text.StringBuilder();
 			if (filenames_item.Active && item != null)
-				sb.Append (item.Name + "  -  ");
+				sb.Append (System.IO.Path.GetFileName (item.DefaultVersionUri.LocalPath) + "  -  ");
 
 			sb.AppendFormat (Catalog.GetPluralString ("{0} Photo", "{0} Photos", collection.Count), collection.Count);
 			status_label.Text = sb.ToString ();
@@ -560,6 +575,20 @@ namespace FSpot {
 				if (filenames_item.Active != (bool) val)
 					filenames_item.Active = (bool) val;
 				break;
+
+			case Preferences.VIEWER_TRANSPARENCY:
+				if (val as string == "CHECK_PATTERN")
+					image_view.SetCheckSize (2);
+				else if (val as string == "COLOR")
+					image_view.SetTransparentColor (Preferences.Get(Preferences.VIEWER_TRANS_COLOR) as string);
+				else // NONE
+					image_view.SetTransparentColor (image_view.Style.BaseColors [(int)Gtk.StateType.Normal]);
+				break;
+
+			case Preferences.VIEWER_TRANS_COLOR:
+				if (Preferences.Get(Preferences.VIEWER_TRANSPARENCY) as string == "COLOR")
+					image_view.SetTransparentColor (val as string);
+				break;
 			}
 		}
 
@@ -576,9 +605,16 @@ namespace FSpot {
 
 		public class PreferenceDialog : GladeDialog {
 			[Glade.Widget] private CheckButton interpolation_check;
+			[Glade.Widget] private ColorButton color_button;
+			[Glade.Widget] private RadioButton as_background_radio;
+			[Glade.Widget] private RadioButton as_check_radio;
+			[Glade.Widget] private RadioButton as_color_radio;
+
 			public PreferenceDialog () : base ("viewer_preferences")
 			{
 				this.LoadPreference (Preferences.VIEWER_INTERPOLATION);
+				this.LoadPreference (Preferences.VIEWER_TRANSPARENCY);
+				this.LoadPreference (Preferences.VIEWER_TRANS_COLOR);
 				Preferences.SettingChanged += OnPreferencesChanged;
 				this.Dialog.Destroyed += HandleDestroyed;
 			}
@@ -587,6 +623,25 @@ namespace FSpot {
 			{
 				Preferences.Set (Preferences.VIEWER_INTERPOLATION, interpolation_check.Active);
 			}
+
+			void HandleTransparentColorSet (object sender, System.EventArgs args)
+			{
+				Preferences.Set (Preferences.VIEWER_TRANS_COLOR, 
+						"#" + 
+						(color_button.Color.Red / 256 ).ToString("x").PadLeft (2, '0') +
+						(color_button.Color.Green / 256 ).ToString("x").PadLeft (2, '0') +
+						(color_button.Color.Blue / 256 ).ToString("x").PadLeft (2, '0'));
+			}
+
+			void HandleTransparencyToggled (object sender, System.EventArgs args)
+			{
+				if (as_background_radio.Active)
+					Preferences.Set (Preferences.VIEWER_TRANSPARENCY, "NONE");
+				else if (as_check_radio.Active)
+					Preferences.Set (Preferences.VIEWER_TRANSPARENCY, "CHECK_PATTERN");
+				else if (as_color_radio.Active)
+					Preferences.Set (Preferences.VIEWER_TRANSPARENCY, "COLOR");
+			}
 			
 			static PreferenceDialog prefs;
 			public static void Show ()
@@ -621,7 +676,26 @@ namespace FSpot {
 			
 				switch (key) {
 				case Preferences.VIEWER_INTERPOLATION:
-						interpolation_check.Active = (bool) val;
+					interpolation_check.Active = (bool) val;
+					break;
+				case Preferences.VIEWER_TRANSPARENCY:
+					switch ((string) val) {
+					case "COLOR":
+						as_color_radio.Active = true;
+						break;
+					case "CHECK_PATTERN":
+						as_check_radio.Active = true;
+						break;
+					default: //NONE
+						as_background_radio.Active = true;
+						break;
+					}
+					break;
+				case Preferences.VIEWER_TRANS_COLOR:
+					color_button.Color = new Gdk.Color (
+						Byte.Parse ((val as string).Substring (1,2), System.Globalization.NumberStyles.AllowHexSpecifier),
+						Byte.Parse ((val as string).Substring (3,2), System.Globalization.NumberStyles.AllowHexSpecifier),
+						Byte.Parse ((val as string).Substring (5,2), System.Globalization.NumberStyles.AllowHexSpecifier));
 					break;
 				}
 			}
