@@ -1,7 +1,7 @@
 using System;
 using System.Collections;
 using System.Text;
-using Mono.Posix;
+using Mono.Unix;
 using Gtk;
 using Gdk;
 
@@ -9,161 +9,161 @@ namespace FSpot.Query
 {
 	public class LiteralPopup
 	{
-		private Literal literal;
-
+		//private Literal literal;
+		
 		public void Activate (Gdk.EventButton eb, Literal literal)
-        {
-            Activate (eb, literal, new Gtk.Menu (), true);
-        }
-
+		{
+			Activate (eb, literal, new Gtk.Menu (), true);
+		}
+		
 		public void Activate (Gdk.EventButton eb, Literal literal, Gtk.Menu popup_menu, bool is_popup)
 		{
-			this.literal = literal;
-
+			//this.literal = literal;
+			
 			/*MenuItem attach_item = new MenuItem (Catalog.GetString ("Find With"));
 			TagMenu attach_menu = new TagMenu (attach_item, MainWindow.Toplevel.Database.Tags);
 			attach_menu.TagSelected += literal.HandleAttachTagCommand;
 			attach_item.ShowAll ();
 			popup_menu.Append (attach_item);*/
 			
-            if (literal.IsNegated) {
-                GtkUtil.MakeMenuItem (popup_menu,
-                    Catalog.GetString ("Include"),
-                    "gtk-cancel",
-                    new EventHandler (literal.HandleToggleNegatedCommand),
-                    true);
-            } else {
-                GtkUtil.MakeMenuItem (popup_menu,
-                    Catalog.GetString ("Exclude"),
-                    "gtk-delete",
-                    new EventHandler (literal.HandleToggleNegatedCommand),
-                    true);
-            }
+			if (literal.IsNegated) {
+				GtkUtil.MakeMenuItem (popup_menu,
+						      Catalog.GetString ("Include"),
+						      "gtk-cancel",
+						      new EventHandler (literal.HandleToggleNegatedCommand),
+						      true);
+			} else {
+				GtkUtil.MakeMenuItem (popup_menu,
+						      Catalog.GetString ("Exclude"),
+						      "gtk-delete",
+						      new EventHandler (literal.HandleToggleNegatedCommand),
+						      true);
+			}
 			
 			GtkUtil.MakeMenuItem (popup_menu, Catalog.GetString ("Remove"),
-                "gtk-remove",
-                new EventHandler (literal.HandleRemoveCommand),
-                true);
-
-            if (is_popup) {
-                if (eb != null)
-                    popup_menu.Popup (null, null, null, eb.Button, eb.Time);
-                else
-                    popup_menu.Popup (null, null, null, 0, Gtk.Global.CurrentEventTime);
-            }
+					      "gtk-remove",
+					      new EventHandler (literal.HandleRemoveCommand),
+					      true);
+			
+			if (is_popup) {
+				if (eb != null)
+					popup_menu.Popup (null, null, null, eb.Button, eb.Time);
+				else
+					popup_menu.Popup (null, null, null, 0, Gtk.Global.CurrentEventTime);
+			}
 		}
 	}
-
+	
 	public class LiteralMenu : Menu
 	{
 		private LiteralPopup popup;
-        private Literal literal;
-
+		private Literal literal;
+		
 		public LiteralMenu (MenuItem item, Literal literal)
 		{
 			popup = new LiteralPopup ();
-
-            this.literal = literal;
-
+			
+			this.literal = literal;
+			
 			item.Submenu = this;
 			item.Activated += HandlePopulate;
 		}
-
+		
 		private void HandlePopulate (object obj, EventArgs args)
 		{
 			foreach (Widget child in Children) {
 				Remove (child);
 				child.Destroy ();
 			}
-
+			
 			popup.Activate (null, literal, this, false);
 		}
 	}
-
-    public static class TermMenuItem
-    {
-        public static void Create (Tag [] tags, Gtk.Menu menu)
-        {
-            Gtk.MenuItem item = new Gtk.MenuItem (String.Format (Catalog.GetPluralString ("Find _With", "Find _With", tags.Length), tags.Length));
-
-            Gtk.Menu submenu = GetSubmenu (tags);
-            if (submenu == null)
+	
+	public static class TermMenuItem
+	{
+		public static void Create (Tag [] tags, Gtk.Menu menu)
+		{
+			Gtk.MenuItem item = new Gtk.MenuItem (String.Format (Catalog.GetPluralString ("Find _With", "Find _With", tags.Length), tags.Length));
+			
+			Gtk.Menu submenu = GetSubmenu (tags);
+			if (submenu == null)
                 item.Sensitive = false;
-            else
-                item.Submenu = submenu;
-
-            menu.Append (item);
-            item.Show ();
-        }
-
-        public static Gtk.Menu GetSubmenu (Tag [] tags)
-        {
-            Tag single_tag = null;
-            if (tags != null && tags.Length == 1)
-                single_tag = tags[0];
-
-            //Console.WriteLine ("creating find with menu item");
-            if (LogicWidget.Root == null || LogicWidget.Root.SubTerms.Count == 0) {
-                //Console.WriteLine ("root is null or has no terms");
-                return null;
-            } else {
-                //Console.WriteLine ("root is not null and has terms");
-                Gtk.Menu m = new Gtk.Menu ();
-
-                Gtk.MenuItem all_item = GtkUtil.MakeMenuItem (m, Catalog.GetString ("All"), new EventHandler (MainWindow.Toplevel.HandleRequireTag));
-                GtkUtil.MakeMenuSeparator (m);
-
-                int sensitive_items = 0;
-                foreach (LogicTerm term in LogicWidget.Root.SubTerms) {
-                    ArrayList term_parts = new ArrayList ();
-
-                    bool contains_tag = AppendTerm (term_parts, term, single_tag);
-
-                    string name = "_" + String.Join (", ", (string []) term_parts.ToArray (typeof(string)));
-
-                    Gtk.MenuItem item = GtkUtil.MakeMenuItem (m, name, new EventHandler (MainWindow.Toplevel.HandleAddTagToTerm));
-                    item.Sensitive = !contains_tag;
-
-                    if (!contains_tag)
-                        sensitive_items++;
-                }
-
-                if (sensitive_items == 0)
-                    all_item.Sensitive = false;
-
-                return m;
-            }
-        }
-
-        private static bool AppendTerm (ArrayList parts, LogicTerm term, Tag single_tag)
-        {
-            bool tag_matches = false;
-            if (term != null) {
-                Literal literal = term as Literal;
-                if (literal != null) {
-                    if (literal.Tag == single_tag)
-                        tag_matches = true;
-
-                    if (literal.IsNegated)
-                        parts.Add (String.Format (Catalog.GetString ("Not {0}"), literal.Tag.Name));
-                    else
-                        parts.Add (literal.Tag.Name);
-                } else {
-                    foreach (LogicTerm subterm in term.SubTerms) {
-                        tag_matches |= AppendTerm (parts, subterm, single_tag);
-                    }
-                }
-            }
-
-            return tag_matches;
-        }
-    }
+			else
+				item.Submenu = submenu;
+			
+			menu.Append (item);
+			item.Show ();
+		}
+		
+		public static Gtk.Menu GetSubmenu (Tag [] tags)
+		{
+			Tag single_tag = null;
+			if (tags != null && tags.Length == 1)
+				single_tag = tags[0];
+			
+			//Console.WriteLine ("creating find with menu item");
+			if (LogicWidget.Root == null || LogicWidget.Root.SubTerms.Count == 0) {
+				//Console.WriteLine ("root is null or has no terms");
+				return null;
+			} else {
+				//Console.WriteLine ("root is not null and has terms");
+				Gtk.Menu m = new Gtk.Menu ();
+				
+				Gtk.MenuItem all_item = GtkUtil.MakeMenuItem (m, Catalog.GetString ("All"), new EventHandler (MainWindow.Toplevel.HandleRequireTag));
+				GtkUtil.MakeMenuSeparator (m);
+				
+				int sensitive_items = 0;
+				foreach (LogicTerm term in LogicWidget.Root.SubTerms) {
+					ArrayList term_parts = new ArrayList ();
+					
+					bool contains_tag = AppendTerm (term_parts, term, single_tag);
+					
+					string name = "_" + String.Join (", ", (string []) term_parts.ToArray (typeof(string)));
+					
+					Gtk.MenuItem item = GtkUtil.MakeMenuItem (m, name, new EventHandler (MainWindow.Toplevel.HandleAddTagToTerm));
+					item.Sensitive = !contains_tag;
+					
+					if (!contains_tag)
+						sensitive_items++;
+				}
+				
+				if (sensitive_items == 0)
+					all_item.Sensitive = false;
+				
+				return m;
+			}
+		}
+		
+		private static bool AppendTerm (ArrayList parts, LogicTerm term, Tag single_tag)
+		{
+			bool tag_matches = false;
+			if (term != null) {
+				Literal literal = term as Literal;
+				if (literal != null) {
+					if (literal.Tag == single_tag)
+						tag_matches = true;
+					
+					if (literal.IsNegated)
+						parts.Add (String.Format (Catalog.GetString ("Not {0}"), literal.Tag.Name));
+					else
+						parts.Add (literal.Tag.Name);
+				} else {
+					foreach (LogicTerm subterm in term.SubTerms) {
+						tag_matches |= AppendTerm (parts, subterm, single_tag);
+					}
+				}
+			}
+			
+			return tag_matches;
+		}
+	}
 
 	public abstract class LogicTerm {
 		private ArrayList sub_terms = new ArrayList ();
 		private LogicTerm parent = null;
 		private string separator;
-
+		
 		protected Tag tag = null;
 
 		public ArrayList SubTerms {
@@ -171,7 +171,7 @@ namespace FSpot.Query
 				return sub_terms;
 			}
 		}
-
+		
 		public LogicTerm (LogicTerm parent, Literal after)
 		{
 			this.parent = parent;
@@ -183,7 +183,7 @@ namespace FSpot.Query
 					parent.SubTerms.Insert (parent.SubTerms.IndexOf (after) + 1, this);
 			}
 		}
-
+		
 		/** Properties **/
 		public bool HasMultiple {
 			get {
@@ -555,10 +555,15 @@ namespace FSpot.Query
 			get {
 				if (widget != null)
 					return widget;
-
+				
 				container = new EventBox ();
-
-				label = new Label ("<u>" + tag.Name + "</u>");
+				((EventBox)container).VisibleWindow = false;
+				box = new HBox ();
+				
+				handle_box = new LiteralBox ();
+				handle_box.BorderWidth = 1;
+				
+				label = new Label (tag.Name);
 				label.UseMarkup = true;
 
 				image = new Gtk.Image (NormalIcon);
@@ -583,6 +588,8 @@ namespace FSpot.Query
 				
 				// Setup this widget as a drag destination (so tags can be added to our parent's LogicTerm)
 				container.DragDataReceived	+= HandleDragDataReceived;
+                container.DragMotion += HandleDragMotion;
+                container.DragLeave += HandleDragLeave;
 
 				Gtk.Drag.DestSet (container, DestDefaults.All, tag_dest_target_table, 
 						  DragAction.Copy | DragAction.Move ); 
@@ -593,12 +600,17 @@ namespace FSpot.Query
 				image.Show ();
 
 				if (tag.Icon == null) {
-					container.Add (label);
+					handle_box.Add (label);
 				} else {
-					container.Add (image);
+					handle_box.Add (image);
 				}
 
-				container.Show ();
+				handle_box.Show ();
+
+                box.Add (handle_box);
+                box.Show ();
+
+                container.Add (box);
 
 				widget = container;
 
@@ -647,7 +659,7 @@ namespace FSpot.Query
 				image.Pixbuf = NegatedIcon;
 			} else {
 				tips.SetTip (widget, tag.Name, null);
-				label.Text = "<u>" + tag.Name + "</u>";
+				label.Text = tag.Name;
 				image.Pixbuf = NormalIcon;
 			}
 
@@ -876,6 +888,26 @@ namespace FSpot.Query
 			}
 		}
 
+		private bool preview = false;
+		private Gtk.Widget preview_widget;
+		private void HandleDragMotion (object o, DragMotionArgs args)
+		{
+			if (!preview) {
+				if (preview_widget == null) {
+					preview_widget = new Gtk.Label (" | ");
+					box.Add (preview_widget);
+				}
+				
+				preview_widget.Show ();
+			}
+		}
+		
+		private void HandleDragLeave (object o, EventArgs args)
+		{
+			preview = false;
+			preview_widget.Hide ();
+		}
+		
 		public void HandleToggleNegatedCommand (object o, EventArgs args)
 		{
 			IsNegated = !IsNegated;
@@ -919,7 +951,9 @@ namespace FSpot.Query
 
 		private static ArrayList focusedLiterals = new ArrayList();
 		private static ArrayList hiddenWidgets = new ArrayList();
-		private Gtk.EventBox container;
+		private Gtk.Container container;
+		private LiteralBox handle_box;
+		private Gtk.Box box;
 		private Gtk.Image image;
 		private Gtk.Label label;
 		
@@ -957,6 +991,52 @@ namespace FSpot.Query
 		public event LiteralsMovedHandler LiteralsMoved;
 	}
 
+    public class LiteralBox : VBox {
+        private GrabHandle handle;
+
+        public LiteralBox () : base ()
+        {
+            handle = new GrabHandle (24, 8);
+
+            PackEnd (handle, false, false, 0);
+
+            Show ();
+        }
+    }
+
+    public class GrabHandle : DrawingArea {
+        public GrabHandle (int w, int h) : base ()
+        {
+            Size (w, h);
+            Orientation = Gtk.Orientation.Horizontal;
+            Show ();
+        }
+
+        private Gtk.Orientation orientation;
+        public Gtk.Orientation Orientation {
+            get { return orientation; }
+            set { orientation = value; }
+        }
+
+        protected override bool OnExposeEvent (Gdk.EventExpose evnt)
+        {
+            bool ret = base.OnExposeEvent(evnt);
+
+            if (evnt.Window != GdkWindow) {
+                return ret;
+            }
+
+            Gtk.Style.PaintHandle(Style, GdkWindow, State, ShadowType.In,
+                evnt.Area, this, "entry", 0, 0, Allocation.Width, Allocation.Height, Orientation);
+
+            //(Style, GdkWindow, StateType.Normal, ShadowType.In,
+            //evnt.Area, this, "entry", 0, y_mid - y_offset, Allocation.Width, 
+            //Height + (y_offset * 2));
+
+            return ret;
+        }
+    }
+
 	public class LogicWidget : HBox {
 		private PhotoQuery query;
 		private TagSelectionWidget tag_selection_widget;
@@ -972,8 +1052,6 @@ namespace FSpot.Query
 		private bool preventUpdate = false;
 		private bool preview = false;
 
-		private ArrayList widgets = new ArrayList ();
-
 		public event EventHandler Changed;
 		
 		public static LogicTerm Root
@@ -1023,10 +1101,11 @@ namespace FSpot.Query
 			preview = false;
 
 			rootAdd = new Gtk.EventBox ();
+			rootAdd.VisibleWindow = false;
 			rootAdd.CanFocus = true;
 			rootAdd.DragMotion		+= HandleDragMotion;
 			rootAdd.DragDataReceived	+= HandleDragDataReceived;
-			rootAdd.DragLeave		+= HandleLeave;
+			rootAdd.DragLeave		+= HandleDragLeave;
 
 			help = new Gtk.Label ("<i>" + Catalog.GetString ("Drag tags here to search for them") + "</i>");
 			help.UseMarkup = true;
@@ -1090,7 +1169,7 @@ namespace FSpot.Query
 			}
 		}
 		
-		private void HandleLeave (object o, EventArgs args)
+		private void HandleDragLeave (object o, EventArgs args)
 		{
 			if (preview && Children.Length > 1) {
 				sepBox.Hide ();
